LIBRARY dllocamlodbc.dll

EXPORTS
	ocamlodbc_HENV_c @1
	ocamlodbc_HDBC_c @2
	ocamlodbc_initDB_c @3
	ocamlodbc_execDB_c @4
	ocamlodbc_itere_execDB_c @5
	ocamlodbc_free_execDB_c @6
	ocamlodbc_get_infoDB_c @7
	ocamlodbc_exitDB_c @8
	ocamlodbc_initDB_driver_c @9
